@@ -4,7 +4,7 @@ import (
 	"fmt"
 	"github.com/daveshanley/vacuum/model"
 	vacuumUtils "github.com/daveshanley/vacuum/utils"
-	"gopkg.in/yaml.v3"
+	"go.yaml.in/yaml/v4"
 )
 
 // PolymorphicOneOf checks that there is no polymorphism used, in particular 'anyOf'
@@ -14,10 +14,15 @@ type PolymorphicOneOf struct {
 // GetSchema returns a model.RuleFunctionSchema defining the schema of the PolymorphicOneOf rule.
 func (pm PolymorphicOneOf) GetSchema() model.RuleFunctionSchema {
 	return model.RuleFunctionSchema{
-		Name: "polymorphic_oneOf",
+		Name: "oasPolymorphicOneOf",
 	}
 }
 
+// GetCategory returns the category of the PolymorphicOneOf rule.
+func (pm PolymorphicOneOf) GetCategory() string {
+	return model.FunctionCategoryOpenAPI
+}
+
 // RunRule will execute the PolymorphicOneOf rule, based on supplied context and a supplied []*yaml.Node slice.
 func (pm PolymorphicOneOf) RunRule(nodes []*yaml.Node, context model.RuleFunctionContext) []model.RuleFunctionResult {
 
