@@ -6,8 +6,8 @@ package owasp
 import (
 	"github.com/daveshanley/vacuum/model"
 	"github.com/daveshanley/vacuum/utils"
-	"github.com/pb33f/doctor/model/high/base"
-	"gopkg.in/yaml.v3"
+	"github.com/pb33f/doctor/model/high/v3"
+	"go.yaml.in/yaml/v4"
 	"slices"
 )
 
@@ -15,7 +15,12 @@ type ArrayLimit struct{}
 
 // GetSchema returns a model.RuleFunctionSchema defining the schema of the DefineError rule.
 func (ar ArrayLimit) GetSchema() model.RuleFunctionSchema {
-	return model.RuleFunctionSchema{Name: "array_limit"}
+	return model.RuleFunctionSchema{Name: "owaspArrayLimit"}
+}
+
+// GetCategory returns the category of the ArrayLimit rule.
+func (ar ArrayLimit) GetCategory() string {
+	return model.FunctionCategoryOWASP
 }
 
 // RunRule will execute the DefineError rule, based on supplied context and a supplied []*yaml.Node slice.
@@ -31,14 +36,24 @@ func (ar ArrayLimit) RunRule(_ []*yaml.Node, context model.RuleFunctionContext)
 		if slices.Contains(schema.Value.Type, "array") {
 			if schema.Value.MaxItems == nil {
 				node := schema.Value.GoLow().Type.KeyNode
+				valueNode := schema.Value.GoLow().Type.ValueNode
+
+				// Find all locations where this schema appears
+				locatedPath, allPaths := LocateSchemaPropertyPaths(context, schema, node, valueNode)
+
 				result := model.RuleFunctionResult{
 					Message:   utils.SuppliedOrDefault(context.Rule.Message, "schema of type `array` must specify `maxItems`"),
 					StartNode: node,
 					EndNode:   utils.BuildEndNode(node),
-					Path:      schema.GenerateJSONPath(),
+					Path:      locatedPath,
 					Rule:      context.Rule,
 				}
-				schema.AddRuleFunctionResult(base.ConvertRuleResult(&result))
+
+				// Set the Paths array if there are multiple locations
+				if len(allPaths) > 1 {
+					result.Paths = allPaths
+				}
+				schema.AddRuleFunctionResult(v3.ConvertRuleResult(&result))
 				results = append(results, result)
 			}
 		}
