@@ -17,15 +17,17 @@ package languageserver
 
 import (
 	"fmt"
-	"github.com/daveshanley/vacuum/model"
-	"github.com/daveshanley/vacuum/motor"
-	"github.com/daveshanley/vacuum/utils"
-	"github.com/tliron/glsp"
-	protocol "github.com/tliron/glsp/protocol_3_16"
-	glspserv "github.com/tliron/glsp/server"
 	"path/filepath"
 	"strings"
 	"time"
+
+	"github.com/daveshanley/vacuum/model"
+	"github.com/daveshanley/vacuum/motor"
+	"github.com/daveshanley/vacuum/utils"
+	"github.com/spf13/viper"
+	"github.com/tliron/glsp"
+	protocol "github.com/tliron/glsp/protocol_3_16"
+	glspserv "github.com/tliron/glsp/server"
 )
 
 var serverName = "vacuum"
@@ -45,7 +47,7 @@ func NewServer(version string, lintRequest *utils.LintFileRequest) *ServerState
 		lintRequest:   lintRequest,
 		documentStore: newDocumentStore(),
 	}
-	handler.Initialize = func(context *glsp.Context, params *protocol.InitializeParams) (interface{}, error) {
+	handler.Initialize = func(context *glsp.Context, params *protocol.InitializeParams) (any, error) {
 		if params.Trace != nil {
 			protocol.SetTraceValue(*params.Trace)
 		}
@@ -53,6 +55,12 @@ func NewServer(version string, lintRequest *utils.LintFileRequest) *ServerState
 		serverCapabilities := handler.CreateServerCapabilities()
 		serverCapabilities.TextDocumentSync = protocol.TextDocumentSyncKindIncremental
 		serverCapabilities.CompletionProvider = &protocol.CompletionOptions{}
+		serverCapabilities.CodeActionProvider = &protocol.CodeActionOptions{
+			CodeActionKinds: []protocol.CodeActionKind{protocol.CodeActionKindQuickFix},
+		}
+		serverCapabilities.ExecuteCommandProvider = &protocol.ExecuteCommandOptions{
+			Commands: []string{"vacuum.openUrl"},
+		}
 
 		return protocol.InitializeResult{
 			Capabilities: serverCapabilities,
@@ -68,7 +76,7 @@ func NewServer(version string, lintRequest *utils.LintFileRequest) *ServerState
 	}
 	handler.TextDocumentDidOpen = func(context *glsp.Context, params *protocol.DidOpenTextDocumentParams) error {
 		doc := state.documentStore.Add(params.TextDocument.URI, params.TextDocument.Text)
-		state.runDiagnostic(doc, context.Notify, false)
+		state.runDiagnostic(doc, context.Notify)
 		return nil
 	}
 	handler.TextDocumentDidChange = func(context *glsp.Context, params *protocol.DidChangeTextDocumentParams) error {
@@ -85,7 +93,7 @@ func NewServer(version string, lintRequest *utils.LintFileRequest) *ServerState
 				doc.Content = c.Text
 			}
 		}
-		state.runDiagnostic(doc, context.Notify, true)
+		state.runDiagnostic(doc, context.Notify)
 		return nil
 	}
 
@@ -97,45 +105,93 @@ func NewServer(version string, lintRequest *utils.LintFileRequest) *ServerState
 	handler.TextDocumentCompletion = func(context *glsp.Context, params *protocol.CompletionParams) (any, error) {
 		return nil, nil
 	}
+
+	handler.TextDocumentCodeAction = func(context *glsp.Context, params *protocol.CodeActionParams) (any, error) {
+		var actions []protocol.CodeAction
+		
+		for _, diagnostic := range params.Context.Diagnostics {
+			if diagnostic.CodeDescription != nil && diagnostic.CodeDescription.HRef != "" {
+				quickFixKind := protocol.CodeActionKindQuickFix
+				actions = append(actions, protocol.CodeAction{
+					Title: "View documentation",
+					Kind:  &quickFixKind,
+					Command: &protocol.Command{
+						Title:     "Open documentation",
+						Command:   "vacuum.openUrl",
+						Arguments: []interface{}{diagnostic.CodeDescription.HRef},
+					},
+				})
+			}
+		}
+		
+		return actions, nil
+	}
+
+	handler.WorkspaceExecuteCommand = func(context *glsp.Context, params *protocol.ExecuteCommandParams) (any, error) {
+		if params.Command == "vacuum.openUrl" && len(params.Arguments) > 0 {
+			if url, ok := params.Arguments[0].(string); ok {
+				utils.OpenURL(url)
+			}
+		}
+		return nil, nil
+	}
 	return state
 }
 
 func (s *ServerState) Run() error {
+	s.initializeConfig()
+
+	viper.OnConfigChange(s.onConfigChange)
+	viper.WatchConfig()
 	return s.server.RunStdio()
 }
 
-func (s *ServerState) runDiagnostic(doc *Document, notify glsp.NotifyFunc, delay bool) {
+func (s *ServerState) runDiagnostic(doc *Document, notify glsp.NotifyFunc) {
 
 	go func() {
+		specFileName := strings.TrimPrefix(doc.URI, "file://")
 
-		if s.lintRequest.BaseFlag == "" {
-			s.lintRequest.BaseFlag = filepath.Dir(strings.TrimPrefix(doc.URI, "file://"))
+		baseForDoc := s.lintRequest.BaseFlag
+		if baseForDoc == "" {
+			baseForDoc = filepath.Dir(specFileName)
+		}
+
+		deepGraph := false
+		if len(s.lintRequest.IgnoredResults) > 0 {
+			deepGraph = true
 		}
 
 		result := motor.ApplyRulesToRuleSet(&motor.RuleSetExecution{
-			RuleSet:                      s.lintRequest.SelectedRS,
-			Timeout:                      time.Duration(s.lintRequest.TimeoutFlag) * time.Second,
-			CustomFunctions:              s.lintRequest.Functions,
-			IgnoreCircularArrayRef:       s.lintRequest.IgnoreArrayCircleRef,
-			IgnoreCircularPolymorphicRef: s.lintRequest.IgnorePolymorphCircleRef,
-			AllowLookup:                  true,
-			Base:                         s.lintRequest.BaseFlag,
-			Spec:                         []byte(doc.Content),
-			SkipDocumentCheck:            s.lintRequest.SkipCheckFlag,
-			Logger:                       s.lintRequest.Logger,
+			RuleSet:                         s.lintRequest.SelectedRS,
+			Spec:                            []byte(doc.Content),
+			SpecFileName:                    specFileName,
+			Timeout:                         time.Duration(s.lintRequest.TimeoutFlag) * time.Second,
+			NodeLookupTimeout:               time.Duration(s.lintRequest.LookupTimeoutFlag) * time.Millisecond,
+			CustomFunctions:                 s.lintRequest.Functions,
+			IgnoreCircularArrayRef:          s.lintRequest.IgnoreArrayCircleRef,
+			IgnoreCircularPolymorphicRef:    s.lintRequest.IgnorePolymorphCircleRef,
+			AllowLookup:                     s.lintRequest.Remote,
+			Base:                            baseForDoc,
+			SkipDocumentCheck:               s.lintRequest.SkipCheckFlag,
+			Logger:                          s.lintRequest.Logger,
+			BuildDeepGraph:                  deepGraph,
+			ExtractReferencesFromExtensions: s.lintRequest.ExtensionRefs,
+			HTTPClientConfig:                s.lintRequest.HTTPClientConfig,
 		})
+
+		filteredResults := utils.FilterIgnoredResults(result.Results, s.lintRequest.IgnoredResults)
+		result.Results = filteredResults
 		diagnostics := ConvertResultsIntoDiagnostics(result)
-		if len(diagnostics) > 0 {
-			go notify(protocol.ServerTextDocumentPublishDiagnostics, protocol.PublishDiagnosticsParams{
-				URI:         doc.URI,
-				Diagnostics: diagnostics,
-			})
-		}
+		go notify(protocol.ServerTextDocumentPublishDiagnostics, protocol.PublishDiagnosticsParams{
+			URI:         doc.URI,
+			Diagnostics: diagnostics,
+		})
 	}()
 }
 
 func ConvertResultsIntoDiagnostics(result *motor.RuleSetExecutionResult) []protocol.Diagnostic {
-	var diagnostics []protocol.Diagnostic
+	diagnostics := []protocol.Diagnostic{}
+
 	for _, vacuumResult := range result.Results {
 		diagnostics = append(diagnostics, ConvertResultIntoDiagnostic(&vacuumResult))
 
@@ -145,23 +201,38 @@ func ConvertResultsIntoDiagnostics(result *motor.RuleSetExecutionResult) []proto
 
 func ConvertResultIntoDiagnostic(vacuumResult *model.RuleFunctionResult) protocol.Diagnostic {
 	severity := GetDiagnosticSeverityFromRule(vacuumResult.Rule)
-	diagnosticErrorHref := fmt.Sprintf("%s/rules/%s/%s", model.WebsiteUrl,
-		strings.ToLower(vacuumResult.Rule.RuleCategory.Id),
-		strings.ReplaceAll(strings.ToLower(vacuumResult.Rule.Id), "$", ""))
 
+	diagnosticErrorHref := fmt.Sprintf("%s/rules/unknown", model.WebsiteUrl)
+	if vacuumResult.Rule.DocumentationURL != "" {
+		diagnosticErrorHref = vacuumResult.Rule.DocumentationURL
+	} else if vacuumResult.Rule.RuleCategory != nil {
+		diagnosticErrorHref = fmt.Sprintf("%s/rules/%s/%s", model.WebsiteUrl,
+			strings.ToLower(vacuumResult.Rule.RuleCategory.Id),
+			strings.ReplaceAll(strings.ToLower(vacuumResult.Rule.Id), "$", ""))
+	}
 	startLine := 1
 	startChar := 1
 	endLine := 1
 	endChar := 1
-	if vacuumResult.StartNode.Line > 0 {
+
+	if vacuumResult.StartNode != nil && vacuumResult.StartNode.Line > 0 {
 		startLine = vacuumResult.StartNode.Line - 1
 		startChar = vacuumResult.StartNode.Column - 1
 	}
-	if vacuumResult.EndNode.Line > 0 {
+	if vacuumResult.EndNode != nil && vacuumResult.EndNode.Line > 0 {
 		endLine = vacuumResult.EndNode.Line - 1
 		endChar = vacuumResult.EndNode.Column - 1
 	}
 
+	// Build comprehensive message with rule details
+	message := vacuumResult.Message
+	if vacuumResult.Rule.Description != "" {
+		message += "\n\nDescription: " + vacuumResult.Rule.Description
+	}
+	if vacuumResult.Rule.HowToFix != "" {
+		message += "\n\nHow to fix: " + vacuumResult.Rule.HowToFix + "\n\nRule ID: " + vacuumResult.Rule.Id + "\n"
+	}
+
 	return protocol.Diagnostic{
 		Range: protocol.Range{
 			Start: protocol.Position{Line: protocol.UInteger(startLine),
@@ -173,7 +244,7 @@ func ConvertResultIntoDiagnostic(vacuumResult *model.RuleFunctionResult) protoco
 		Source:          &serverName,
 		Code:            &protocol.IntegerOrString{Value: vacuumResult.Rule.Id},
 		CodeDescription: &protocol.CodeDescription{HRef: diagnosticErrorHref},
-		Message:         vacuumResult.Message,
+		Message:         message,
 	}
 }
 
