@@ -2,17 +2,21 @@ package motor
 
 import (
 	"fmt"
-	"github.com/daveshanley/vacuum/plugin"
+	"log"
 	"os"
+	"path/filepath"
+	"strings"
+	"sync"
 	"testing"
 	"time"
 
 	"github.com/daveshanley/vacuum/model"
+	"github.com/daveshanley/vacuum/plugin"
 	"github.com/daveshanley/vacuum/rulesets"
 	"github.com/pb33f/libopenapi"
 	"github.com/pb33f/libopenapi/datamodel"
 	"github.com/stretchr/testify/assert"
-	"gopkg.in/yaml.v3"
+	"go.yaml.in/yaml/v4"
 )
 
 func TestApplyRules_PostResponseSuccess(t *testing.T) {
@@ -104,7 +108,7 @@ func TestApplyRules_PostResponseFailure(t *testing.T) {
       "type": "style",
       "given": "$.paths.*.post.responses",
       "then": {
-        "function": "post-response-success",
+        "function": "postResponseSuccess",
 		"functionOptions" : { 
 			"properties": [
 				"900", "300", "750", "600" 
@@ -183,7 +187,7 @@ func TestApplyRules_LengthTestFail(t *testing.T) {
       "description": "this is a test for checking the length function",
       "recommended": true,
       "type": "style",
-      "given": "$.paths./burgers.post.requestBody.content.application/json",
+      "given": "$.paths['/burgers'].post.requestBody.content['application/json']",
       "severity": "%s",
       "then": {
         "function": "length",
@@ -223,7 +227,7 @@ func TestApplyRules_LengthTestSuccess(t *testing.T) {
       "description": "this is a test for checking the length function",
       "recommended": true,
       "type": "style",
-      "given": "$.paths./burgers.post.requestBody.content.application/json",
+      "given": "$.paths['/burgers'].post.requestBody.content['application/json']",
       "severity": "%s",
       "then": {
         "function": "length",
@@ -647,7 +651,7 @@ func TestApplyRules_Length_Description_BadConfig(t *testing.T) {
 	results := ApplyRulesToRuleSet(rse)
 	assert.Len(t, results.Errors, 0)
 	assert.Len(t, results.Results, 1)
-	assert.Equal(t, "'length' needs 'min' or 'max' (or both) properties being set to operate: minimum property number not met (1)",
+	assert.Equal(t, "'length' needs 'min' or 'max' (or both) function options being set to operate: minimum property number not met (1)",
 		results.Results[0].Message)
 
 }
@@ -749,7 +753,7 @@ info:
 	}
 	results := ApplyRulesToRuleSet(rse)
 	assert.Len(t, results.Errors, 0)
-	assert.Equal(t, "Contact details are incomplete: `url` must be set", results.Results[0].Message)
+	assert.Equal(t, "`contact` section must contain a `url`", results.Results[0].Message)
 
 }
 
@@ -775,7 +779,7 @@ info:
 	assert.Len(t, results.Errors, 0)
 
 	assert.NotNil(t, results)
-	assert.Equal(t, "Info section is missing contact details: `contact` must be set", results.Results[0].Message)
+	assert.Equal(t, "`info` section must contain `contact` details", results.Results[0].Message)
 
 }
 
@@ -801,7 +805,7 @@ info:
 	}
 	results := ApplyRulesToRuleSet(rse)
 	assert.Len(t, results.Errors, 0)
-	assert.Equal(t, "Info section is missing a description: `description` must be set", results.Results[0].Message)
+	assert.Equal(t, "`info` section must have a `description`", results.Results[0].Message)
 
 }
 
@@ -828,7 +832,7 @@ info:
 	}
 	results := ApplyRulesToRuleSet(rse)
 	assert.Len(t, results.Errors, 0)
-	assert.Equal(t, "Info section should contain a license: `license` must be set", results.Results[0].Message)
+	assert.Equal(t, "`info` section must contain a `license`", results.Results[0].Message)
 
 }
 
@@ -859,7 +863,7 @@ info:
 	assert.Len(t, results.Errors, 0)
 
 	assert.NotNil(t, results)
-	assert.Equal(t, "License should contain a URL: `url` must be set", results.Results[0].Message)
+	assert.Equal(t, "`license` section must contain a `url` or an `identifier`", results.Results[0].Message)
 }
 
 func TestRuleSet_NoEvalInMarkdown(t *testing.T) {
@@ -1007,7 +1011,7 @@ components:
 	assert.Len(t, results.Errors, 0)
 
 	assert.NotNil(t, results)
-	assert.Equal(t, "Top level spec `tags` must not be empty, and must be an array: expected array, but got string",
+	assert.Equal(t, "Top level spec `tags` must not be empty, and must be an array: got string, want array",
 		results.Results[0].Message)
 
 }
@@ -1044,7 +1048,7 @@ components:
 	assert.Len(t, results.Errors, 0)
 
 	assert.NotNil(t, results)
-	assert.Equal(t, "Top level spec `tags` must not be empty, and must be an array: expected object, but got string",
+	assert.Equal(t, "Top level spec `tags` must not be empty, and must be an array: got string, want object",
 		results.Results[0].Message)
 
 }
@@ -1260,7 +1264,8 @@ tags:
 	results := ApplyRulesToRuleSet(rse)
 	assert.Len(t, results.Errors, 0)
 	assert.Len(t, results.Results, 1)
-	assert.Equal(t, "Tag must have a description defined: `description` must be set", results.Results[0].Message)
+	assert.Equal(t, "tag `cinnamon` must have a description", results.Results[0].Message)
+	assert.Equal(t, 5, results.Results[0].StartNode.Line)
 
 }
 
@@ -1574,6 +1579,10 @@ func (t *testRule) GetSchema() model.RuleFunctionSchema {
 	}
 }
 
+func (t *testRule) GetCategory() string {
+	return model.CategoryValidation
+}
+
 func (t *testRule) RunRule(nodes []*yaml.Node,
 	context model.RuleFunctionContext,
 ) []model.RuleFunctionResult {
@@ -1609,9 +1618,10 @@ components:
     none:
       type: int`)
 
-	panicRan := false
+	panicChan := make(chan bool)
+
 	saveMePlease := func(r any) {
-		panicRan = true
+		panicChan <- true
 	}
 
 	ApplyRulesToRuleSet(
@@ -1625,8 +1635,7 @@ components:
 		})
 
 	//nolint:staticcheck // ignore this linting issue, it's not a bug, it's on purpose.
-	time.Sleep(100)
-	assert.True(t, panicRan)
+	<-panicChan
 
 }
 
@@ -1690,13 +1699,17 @@ components:
 	assert.Len(t, results.Errors, 0)
 
 	assert.NotNil(t, results)
-	assert.Equal(t, "component '#/components/schemas/one' does not exist in the specification",
+	assert.Equal(t, "component `#/components/schemas/one` does not exist in the specification",
 		results.Results[0].Message)
 	assert.Equal(t, "resolving-references", results.Results[0].RuleId)
 }
 
 type testRuleNotResolved struct{}
 
+func (t *testRuleNotResolved) GetCategory() string {
+	return model.CategoryValidation
+}
+
 func (r *testRuleNotResolved) RunRule(nodes []*yaml.Node, context model.RuleFunctionContext) []model.RuleFunctionResult {
 	paths := context.Index.GetAllPaths()
 	oneRef := paths["/one"]["get"].Node.Content[1].Content[1].Content[1].Content[1].Content[1].Content[0].Value
@@ -1720,6 +1733,10 @@ func (r *testRuleNotResolved) GetSchema() model.RuleFunctionSchema {
 
 type testRuleResolved struct{}
 
+func (r *testRuleResolved) GetCategory() string {
+	return model.CategoryValidation
+}
+
 func (r *testRuleResolved) RunRule(nodes []*yaml.Node, context model.RuleFunctionContext) []model.RuleFunctionResult {
 	paths := context.Index.GetAllPaths()
 	oneRef := paths["/one"]["get"].Node.Content[1].Content[1].Content[1].Content[1].Content[1].Content[0].Value
@@ -1832,7 +1849,8 @@ components:
 				},
 			},
 		},
-		Document: doc,
+		NodeLookupTimeout: 10 * time.Duration(time.Millisecond), // override the default 500ms.
+		Document:          doc,
 		CustomFunctions: map[string]model.RuleFunction{
 			"resolved": &testRuleResolved{},
 		},
@@ -1843,6 +1861,59 @@ components:
 	assert.Nil(t, results.Results)
 }
 
+func TestRuleSet_TestDocument_SetNodeLookupTimeout(t *testing.T) {
+
+	yml := `openapi: 3.1.0
+paths:
+  /one:
+    get:
+      responses:
+        '200':
+          content:
+            application/json:
+              schema:
+                $ref: '#/components/schemas/one'
+components:
+  schemas:
+    one:
+      type: string`
+
+	config := datamodel.NewDocumentConfiguration()
+
+	doc, err := libopenapi.NewDocumentWithConfiguration([]byte(yml), config)
+	if err != nil {
+		panic(err)
+	}
+
+	ex := &RuleSetExecution{
+		RuleSet: &rulesets.RuleSet{
+			Rules: map[string]*model.Rule{
+				"test": {
+					Id:           "test",
+					Resolved:     true,
+					Given:        "$..paths",
+					RuleCategory: model.RuleCategories[model.CategoryValidation],
+					Type:         rulesets.Validation,
+					Severity:     model.SeverityError,
+					Then: model.RuleAction{
+						Function: "resolved",
+					},
+				},
+			},
+		},
+		NodeLookupTimeout: 1 * time.Duration(time.Nanosecond), // override the default 500ms to 1ns.(1ns is not enough to resolve the node)
+		Document:          doc,
+		CustomFunctions: map[string]model.RuleFunction{
+			"resolved": &testRuleResolved{},
+		},
+	}
+
+	results := ApplyRulesToRuleSet(ex)
+	assert.Len(t, results.Errors, 1)
+	assert.Nil(t, results.Results)
+	assert.Equal(t, "node lookup timeout exceeded (1ns)", results.Errors[0].Error())
+}
+
 func TestRuleSet_InfiniteCircularLoop(t *testing.T) {
 
 	yml := `openapi: 3.1.0
@@ -2025,7 +2096,7 @@ func TestApplyRules_TestRules_Custom_JS_Function_CustomDoc(t *testing.T) {
     given: $.custom
     severity: error
     then:
-      function: check_for_name_and_id
+      function: checkForNameAndId
 `
 
 	defaultRuleSets := rulesets.BuildDefaultRuleSets()
@@ -2072,7 +2143,7 @@ func TestApplyRules_TestRules_Custom_JS_Function_CustomDoc_CoreFunction(t *testi
     given: $
     severity: error
     then:
-      function: use_core_function
+      function: useCoreFunction
       field: "custom"
 `
 
@@ -2119,7 +2190,7 @@ func TestApplyRules_TestRules_Custom_JS_Function_CustomDoc_CheckPaths(t *testing
     given: $.paths
     severity: error
     then:
-      function: check_single_path
+      function: checkSinglePath
 `
 
 	defaultRuleSets := rulesets.BuildDefaultRuleSets()
@@ -2157,7 +2228,7 @@ paths:
 	results := ApplyRulesToRuleSet(rse)
 
 	assert.Len(t, results.Results, 1)
-	assert.Equal(t, "more than a single path exists, there are 3", results.Results[0].Message)
+	assert.Equal(t, "More than a single path exists, found 3 paths", results.Results[0].Message)
 }
 
 func TestApplyRules_TestRules_Custom_JS_Function_CustomDoc_CoreFunction_FunctionOptions(t *testing.T) {
@@ -2168,7 +2239,7 @@ func TestApplyRules_TestRules_Custom_JS_Function_CustomDoc_CoreFunction_Function
     given: $
     severity: error
     then:
-      function: use_function_options
+      function: useFunctionOptions
       field: "custom"
       functionOptions:
          someOption: "someValue"
@@ -2281,3 +2352,228 @@ func Benchmark_StripeSpecAgainstDefaultRuleSet(b *testing.B) {
 		assert.NotNil(b, results)
 	}
 }
+
+func Benchmark_PetStoreSpecAgainstDefaultRuleSet(b *testing.B) {
+	m, _ := os.ReadFile("../model/test_files/petstorev3.json")
+	rs := rulesets.BuildDefaultRuleSets()
+	for n := 0; n < b.N; n++ {
+		rse := &RuleSetExecution{
+			RuleSet: rs.GenerateOpenAPIDefaultRuleSet(),
+			Spec:    m,
+		}
+		results := ApplyRulesToRuleSet(rse)
+		assert.Len(b, results.Errors, 0)
+		assert.NotNil(b, results)
+	}
+}
+
+func Test_Issue486(t *testing.T) {
+
+	yml := `openapi: 3.1.0
+paths:
+  /hi:
+    get:
+      requestBody:
+        content:
+          application/json:
+            schema:
+              $ref: "https://raw.githubusercontent.com/OAI/OpenAPI-Specification/refs/heads/main/_archive_/schemas/v3.0/schema.yaml"
+`
+
+	rules := make(map[string]*model.Rule)
+	rules["oas3-host-not-example"] = rulesets.GetOAS3HostNotExampleRule()
+
+	rs := &rulesets.RuleSet{
+		Rules: rules,
+	}
+
+	rse := &RuleSetExecution{
+		RuleSet:     rs,
+		Spec:        []byte(yml),
+		AllowLookup: true,
+	}
+	results := ApplyRulesToRuleSet(rse)
+	assert.Len(t, results.Errors, 0)
+	assert.Len(t, results.Results, 4)
+
+}
+
+func Test_EnsureConsistentDeterministicOutcome_Issue673(t *testing.T) {
+
+	// a glitch in the doctor was causing mayhem. This test ensures that the glitch stays dead.
+	//https://github.com/daveshanley/vacuum/issues/673
+
+	yml := []byte(`openapi: 3.0.3
+
+info:
+  title: API title
+  description: API description
+  version: 0.0.1
+
+servers:
+  - url: http://some.url
+
+tags:
+  - name: tag_a
+
+paths:
+  /a:
+    get:
+      summary: Summary of GET /a
+      description: Description of GET /a
+      tags:
+        - tag_a
+      operationId: operationA
+      responses:
+        "200":
+          description: Description of the 200 response to GET /a
+          content:
+            application/json:
+              schema:
+                $ref: '#/components/schemas/Schema1'
+              examples:
+                Example1:
+                  value:
+                    a: foo
+                    b: bar
+
+components:
+  schemas:
+    Schema1:
+      type: object
+      description: Description of Schema1
+      properties:
+        a:
+          type: string
+          example: fieldA
+        b:
+          type: string
+          example: fieldB
+
+  securitySchemes:
+    Default:
+      description: Description of the securityScheme.
+      type: http
+      scheme: bearer
+
+security:
+  - Default: []
+`)
+
+	run := func(wg *sync.WaitGroup) {
+		docConf := &datamodel.DocumentConfiguration{}
+		d, err := libopenapi.NewDocumentWithConfiguration(yml, docConf)
+		if err != nil {
+			log.Fatalf("error: %v", err)
+		}
+		d.BuildV3Model()
+
+		ex := &RuleSetExecution{
+			RuleSet:           rulesets.BuildDefaultRuleSets().GenerateOpenAPIDefaultRuleSet(),
+			Document:          d,
+			NodeLookupTimeout: 5 * time.Second,  // Increase timeout for CI/CD environments
+			Timeout:           30 * time.Second, // Increase rule timeout for CI/CD environments
+		}
+
+		results := ApplyRulesToRuleSet(ex)
+		assert.Len(t, results.Results, 4)
+		if wg != nil {
+			wg.Done()
+		}
+	}
+
+	// run it a thousand times to ensure that the results are consistent and deterministic.
+	for i := 0; i < 1000; i++ {
+		run(nil)
+	}
+
+	// now run it async
+	var wg sync.WaitGroup
+	wg.Add(1000)
+	for i := 0; i < 1000; i++ {
+		go run(&wg)
+	}
+	wg.Wait()
+}
+
+// TestIssue523_ExtensionReferenceToNonYAMLFile tests that vacuum handles references
+// to non-YAML files (like .go files) in extensions when ext-refs flag is enabled
+// Issue: https://github.com/daveshanley/vacuum/issues/523
+func TestIssue523_ExtensionReferenceToNonYAMLFile(t *testing.T) {
+	// Load the test spec
+	testFile := "../model/test_files/issue523_spec.yaml"
+	specBytes, err := os.ReadFile(testFile)
+	assert.NoError(t, err)
+
+	// Test with ext-refs disabled (default behavior)
+	t.Run("ext-refs disabled", func(t *testing.T) {
+		// Load default rulesets
+		defaultRuleSets := rulesets.BuildDefaultRuleSets()
+
+		// Use the recommended ruleset
+		selectedRS := defaultRuleSets.GenerateOpenAPIRecommendedRuleSet()
+
+		// Set base path to where the files are
+		execution := &RuleSetExecution{
+			RuleSet:                         selectedRS,
+			SpecFileName:                    testFile,
+			Spec:                            specBytes,
+			ExtractReferencesFromExtensions: false, // ext-refs disabled
+			Base:                            filepath.Dir(testFile),
+			AllowLookup:                     true,
+			SilenceLogs:                     true,
+		}
+
+		results := ApplyRulesToRuleSet(execution)
+
+		// With ext-refs disabled, references in extensions should be ignored
+		// so we shouldn't get errors about references in x-codeSamples
+		refErrors := 0
+		for _, res := range results.Results {
+			if res.Rule.Id == "resolving-references" && strings.Contains(res.Message, "issue523") {
+				refErrors++
+			}
+		}
+
+		// When ext-refs is disabled, references in extensions should be ignored
+		assert.Equal(t, 0, refErrors, "Should not have reference errors for files in x-codeSamples when ext-refs is disabled")
+	})
+
+	// Test with ext-refs enabled
+	t.Run("ext-refs enabled", func(t *testing.T) {
+		// Load default rulesets
+		defaultRuleSets := rulesets.BuildDefaultRuleSets()
+
+		// Use the recommended ruleset
+		selectedRS := defaultRuleSets.GenerateOpenAPIRecommendedRuleSet()
+
+		// Set base path to where the files are
+		execution := &RuleSetExecution{
+			RuleSet:                         selectedRS,
+			SpecFileName:                    testFile,
+			Spec:                            specBytes,
+			ExtractReferencesFromExtensions: true, // ext-refs enabled
+			Base:                            filepath.Dir(testFile),
+			AllowLookup:                     true,
+			SilenceLogs:                     true,
+		}
+
+		results := ApplyRulesToRuleSet(execution)
+
+		// With ext-refs enabled and the fix applied, vacuum should now be able to
+		// find and load the referenced files without errors
+		refErrors := 0
+		for _, res := range results.Results {
+			// Look for any errors related to our test files
+			if strings.Contains(res.Message, "issue523_echo.go") || strings.Contains(res.Message, "issue523_test.yaml") {
+				if res.Rule.Id == "resolving-references" || res.Rule.Id == "oas3-unused-component" {
+					refErrors++
+				}
+			}
+		}
+
+		// After the fix, the files should be found and loaded successfully
+		// The rolodex can handle non-YAML files by creating a fake YAML node with the content
+		assert.Equal(t, 0, refErrors, "Should not have reference errors after fix - files are found and loaded")
+	})
+}
