@@ -1,6 +1,13 @@
 // Copyright 2023 Princess Beef Heavy Industries / Dave Shanley.
 // SPDX-License-Identifier: MIT
 
+function getSchema() {
+    return {
+        "name": "useFunctionOptions",
+        "description": "Demonstrates how to use function options passed from rulesets"
+    };
+}
+
 function runRule(input) {
     // extract function options from context
     const functionOptions = context.ruleAction.functionOptions
