@@ -4,9 +4,10 @@
 package rulesets
 
 import (
+	"regexp"
+
 	"github.com/daveshanley/vacuum/model"
 	"github.com/daveshanley/vacuum/parser"
-	"regexp"
 )
 
 // GetContactPropertiesRule will return a rule configured to look at contact properties of a spec.
@@ -17,25 +18,14 @@ func GetContactPropertiesRule() *model.Rule {
 		Id:           ContactProperties,
 		Formats:      model.AllFormats,
 		Description:  "Contact details are incomplete",
-		Given:        "$.info.contact",
+		Given:        "$",
 		Resolved:     true,
 		RuleCategory: model.RuleCategories[model.CategoryInfo],
 		Recommended:  false,
 		Type:         Validation,
 		Severity:     model.SeverityInfo,
-		Then: []model.RuleAction{
-			{
-				Field:    "name",
-				Function: "truthy",
-			},
-			{
-				Field:    "url",
-				Function: "truthy",
-			},
-			{
-				Field:    "email",
-				Function: "truthy",
-			},
+		Then: model.RuleAction{
+			Function: "infoContactProperties",
 		},
 		HowToFix: contactPropertiesFix,
 	}
@@ -49,15 +39,14 @@ func GetInfoContactRule() *model.Rule {
 		Id:           InfoContact,
 		Formats:      model.AllFormats,
 		Description:  "Info section is missing contact details",
-		Given:        "$.info",
+		Given:        "$",
 		Resolved:     true,
 		Recommended:  false,
 		RuleCategory: model.RuleCategories[model.CategoryInfo],
 		Type:         Validation,
 		Severity:     model.SeverityWarn,
 		Then: model.RuleAction{
-			Field:    "contact",
-			Function: "truthy",
+			Function: "infoContact",
 		},
 		HowToFix: contactFix,
 	}
@@ -71,20 +60,39 @@ func GetInfoDescriptionRule() *model.Rule {
 		Id:           InfoDescription,
 		Formats:      model.AllFormats,
 		Description:  "Info section is missing a description",
-		Given:        "$.info",
+		Given:        "$",
 		Resolved:     true,
 		RuleCategory: model.RuleCategories[model.CategoryInfo],
 		Recommended:  true,
 		Type:         Validation,
 		Severity:     model.SeverityError,
 		Then: model.RuleAction{
-			Field:    "description",
-			Function: "truthy",
+			Function: "infoDescription",
 		},
 		HowToFix: infoDescriptionFix,
 	}
 }
 
+// GetInfoLicenseSPDXRule will check that a license either has a URL OR an identifier, not both.
+func GetInfoLicenseSPDXRule() *model.Rule {
+	return &model.Rule{
+		Name:         "Check license object for URL or identifier, but not both",
+		Id:           InfoLicenseSPDX,
+		Formats:      model.AllFormats,
+		Description:  "License section cannot contain both an identifier and a URL, they are mutually exclusive.",
+		Given:        "$",
+		Resolved:     true,
+		RuleCategory: model.RuleCategories[model.CategoryInfo],
+		Recommended:  true,
+		Type:         Validation,
+		Severity:     model.SeverityError,
+		Then: model.RuleAction{
+			Function: "infoLicenseURLSPDX",
+		},
+		HowToFix: infoLicenseSPDXFix,
+	}
+}
+
 // GetInfoLicenseRule will return a rule that uses the truthy function to check if the
 // info object contains a license
 func GetInfoLicenseRule() *model.Rule {
@@ -100,8 +108,7 @@ func GetInfoLicenseRule() *model.Rule {
 		Type:         Style,
 		Severity:     model.SeverityInfo,
 		Then: model.RuleAction{
-			Field:    "license",
-			Function: "truthy",
+			Function: "infoLicense",
 		},
 		HowToFix: infoLicenseFix,
 	}
@@ -122,8 +129,7 @@ func GetInfoLicenseUrlRule() *model.Rule {
 		Type:         Style,
 		Severity:     model.SeverityInfo,
 		Then: model.RuleAction{
-			Field:    "url",
-			Function: "truthy",
+			Function: "infoLicenseUrl",
 		},
 		HowToFix: infoLicenseUrlFix,
 	}
@@ -367,7 +373,7 @@ func GetOAS3HostTrailingSlashRule() *model.Rule {
 	return &model.Rule{
 		Name:         "Check server url for trailing slash",
 		Id:           Oas3HostTrailingSlash,
-		Formats:      model.OAS3Format,
+		Formats:      model.OAS3AllFormat,
 		Description:  "server URL should not contain a trailing slash",
 		Given:        "$.servers[*]",
 		Resolved:     false,
@@ -640,15 +646,14 @@ func GetTagDescriptionRequiredRule() *model.Rule {
 		Id:           TagDescription,
 		Formats:      model.AllFormats,
 		Description:  "Tag must have a description defined",
-		Given:        "$.tags",
+		Given:        "$",
 		Resolved:     true,
 		Recommended:  false,
 		RuleCategory: model.RuleCategories[model.CategoryTags],
 		Type:         Validation,
 		Severity:     model.SeverityWarn,
 		Then: model.RuleAction{
-			Field:    "description",
-			Function: "truthy",
+			Function: "tagDescription",
 		},
 		HowToFix: tagDescriptionRequiredFix,
 	}
@@ -674,13 +679,33 @@ func GetTypedEnumRule() *model.Rule {
 	}
 }
 
+// GetNullableEnumRule checks that nullable enums explicitly contain null in their enum array
+func GetNullableEnumRule() *model.Rule {
+	return &model.Rule{
+		Name:         "Check nullable enums contain null",
+		Id:           NullableEnumRule,
+		Formats:      model.AllFormats,
+		Description:  "Nullable enums must explicitly include null in the enum array",
+		Given:        "$",
+		Resolved:     true,
+		Recommended:  true,
+		RuleCategory: model.RuleCategories[model.CategorySchemas],
+		Type:         Validation,
+		Severity:     model.SeverityWarn,
+		Then: model.RuleAction{
+			Function: "nullableEnum",
+		},
+		HowToFix: nullableEnumFix,
+	}
+}
+
 // GetPathParamsRule checks if path params are valid and defined.
 func GetPathParamsRule() *model.Rule {
 	// add operation tag defined rule
 	return &model.Rule{
 		Name:         "Check path validity and definition",
 		Id:           PathParamsRule,
-		Formats:      model.AllFormats,
+		Formats:      model.OAS3AllFormat,
 		Description:  "Path parameters must be defined and valid.",
 		Given:        "$",
 		Resolved:     true,
@@ -824,8 +849,8 @@ func GetOAS3SchemaRule() *model.Rule {
 	return &model.Rule{
 		Name:         "Check spec is valid OpenAPI 3",
 		Id:           Oas3Schema,
-		Formats:      model.OAS3Format,
-		Description:  "OpenAPI 3 specification is invalid",
+		Formats:      model.OAS3AllFormat,
+		Description:  "OpenAPI 3+ specification is invalid",
 		Given:        "$",
 		Resolved:     false,
 		Recommended:  true,
@@ -966,7 +991,7 @@ func GetNoRefSiblingsRule() *model.Rule {
 	return &model.Rule{
 		Name:         "Check for siblings to $ref values",
 		Id:           NoRefSiblings,
-		Formats:      model.AllFormats,
+		Formats:      model.AllExceptOAS3_1,
 		Description:  "$ref values cannot be placed next to other properties (like a description)",
 		Given:        "$",
 		Resolved:     false,
@@ -981,6 +1006,26 @@ func GetNoRefSiblingsRule() *model.Rule {
 	}
 }
 
+// GetNoRefSiblingsRule // disabled for OAS3.1 as it's allowed there and libopenapi supports it.
+func GetOAS3NoRefSiblingsRule() *model.Rule {
+	return &model.Rule{
+		Name:         "Check for siblings to $ref values",
+		Id:           Oas3NoRefSiblings,
+		Formats:      model.OAS3_1Format,
+		Description:  "`$ref` values cannot be placed next to other properties, except `description` and `summary`",
+		Given:        "$",
+		Resolved:     false,
+		Recommended:  true,
+		RuleCategory: model.RuleCategories[model.CategorySchemas],
+		Type:         Validation,
+		Severity:     model.SeverityError,
+		Then: model.RuleAction{
+			Function: "oasRefSiblings",
+		},
+		HowToFix: oas3noRefSiblingsFix,
+	}
+}
+
 // GetOAS3UnusedComponentRule will check that there aren't any components anywhere that haven't been used.
 func GetOAS3UnusedComponentRule() *model.Rule {
 	return &model.Rule{
@@ -1177,7 +1222,7 @@ func NoAmbiguousPaths() *model.Rule {
 		Type:         Validation,
 		Severity:     model.SeverityError,
 		Then: model.RuleAction{
-			Function: "ambiguousPaths",
+			Function: "noAmbiguousPaths",
 		},
 		HowToFix: ambiguousPathsFix,
 	}
@@ -1230,7 +1275,7 @@ func GetOperationErrorResponseRule() *model.Rule {
 		Id:           OperationErrorResponse,
 		Formats:      model.AllFormats,
 		Description:  "Make sure operations return at least one `4xx` error response to help with bad requests",
-		Given:        "$.paths",
+		Given:        "$",
 		Resolved:     true,
 		Recommended:  false,
 		RuleCategory: model.RuleCategories[model.CategoryOperations],
@@ -1262,3 +1307,166 @@ func GetSchemaTypeCheckRule() *model.Rule {
 		HowToFix: schemaTypeFix,
 	}
 }
+
+// GetMissingTypeRule will check that all schemas and their properties have a type defined
+func GetMissingTypeRule() *model.Rule {
+	return &model.Rule{
+		Name:         "schemas and properties must have a type defined",
+		Id:           OasMissingType,
+		Formats:      model.OAS3AllFormat,
+		Description:  "All schemas and their properties should have a type field (unless using enum, const, or a composition)",
+		Given:        "$",
+		Resolved:     false,
+		Recommended:  true,
+		RuleCategory: model.RuleCategories[model.CategorySchemas],
+		Type:         Validation,
+		Severity:     model.SeverityInfo,
+		Then: model.RuleAction{
+			Function: "missingType",
+		},
+		HowToFix: "Add a `type` field to all schemas and properties. Valid types are: string, number, integer, boolean, array, object, or null.",
+	}
+}
+
+// GetPostSuccessResponseRule will check that all POST operations have a success response defined.
+func GetPostSuccessResponseRule() *model.Rule {
+	opts := make(map[string][]string)
+	opts["properties"] = []string{"2XX", "3XX", "200", "201", "202", "204", "205", "206", "207", "208", "226", "300", "301", "302", "303", "304", "305", "306", "307", "308"}
+	return &model.Rule{
+		Name:         "Check POST operations for success response",
+		Id:           PostResponseSuccess,
+		Formats:      model.OAS3AllFormat,
+		Description:  "POST Operations should have a success response defined",
+		Given:        "$.paths.*.post.responses",
+		Resolved:     false,
+		RuleCategory: model.RuleCategories[model.CategoryOperations],
+		Recommended:  false,
+		Type:         Validation,
+		Severity:     model.SeverityWarn,
+		Then: model.RuleAction{
+			Function:        "postResponseSuccess",
+			FunctionOptions: opts,
+		},
+		HowToFix: postSuccessResponseFix,
+	}
+}
+
+// GetNoRequestBodyRule will check that HTTP GET and DELETE do not accept request bodies
+func GetNoRequestBodyRule() *model.Rule {
+	return &model.Rule{
+		Name:         "Check GET and DELETE methods do not accept request bodies",
+		Id:           NoRequestBody,
+		Formats:      model.OAS3AllFormat,
+		Description:  "HTTP GET and DELETE should not accept request bodies",
+		Given:        "$",
+		Resolved:     false,
+		RuleCategory: model.RuleCategories[model.CategoryOperations],
+		Recommended:  true,
+		Type:         Style,
+		Severity:     model.SeverityWarn,
+		Then: model.RuleAction{
+			Function: "noRequestBody",
+		},
+		HowToFix: noRequestBodyResponseFix,
+	}
+}
+
+// GetPathItemReferencesRule will check that path items have not used references, as they are not allowed.
+func GetPathItemReferencesRule() *model.Rule {
+	return &model.Rule{
+		Name:         "Check path items have not used references",
+		Id:           PathItemReferences,
+		Formats:      model.OAS3AllFormat,
+		Description:  "Path items should not use references ($ref) values. They are technically not allowed.",
+		Given:        "$",
+		Resolved:     false,
+		RuleCategory: model.RuleCategories[model.CategoryOperations],
+		Recommended:  true,
+		Type:         Validation,
+		Severity:     model.SeverityInfo,
+		Then: model.RuleAction{
+			Function: "pathItemReferences",
+		},
+		HowToFix: PathItemReferencesFix,
+	}
+}
+
+// GetDuplicatePathsRule will check for duplicate path definitions in the OpenAPI spec.
+func GetDuplicatePathsRule() *model.Rule {
+	return &model.Rule{
+		Name:         "Check for duplicate paths",
+		Id:           DuplicatePathsRule,
+		Formats:      model.OAS3AllFormat,
+		Description:  "Paths cannot be duplicated; only the last definition will be kept.",
+		Given:        "$",
+		Resolved:     false, // we want raw YAML nodes to detect duplicates before parsing
+		RuleCategory: model.RuleCategories[model.CategoryOperations],
+		Recommended:  true,
+		Type:         Validation,
+		Severity:     model.SeverityError,
+		Then: model.RuleAction{
+			Function: "duplicatePaths",
+		},
+		HowToFix: duplicatePathsFix,
+	}
+}
+
+// GetUnnecessaryCombinatorRule will check for schema combinators with only a single item.
+func GetUnnecessaryCombinatorRule() *model.Rule {
+	return &model.Rule{
+		Name:         "Check for unnecessary combinators",
+		Id:           UnnecessaryCombinatorRule,
+		Formats:      model.OAS3AllFormat,
+		Description:  "Schema combinators (allOf, anyOf, oneOf) with only one item should be replaced with the item directly.",
+		Given:        "$",
+		Resolved:     true,
+		RuleCategory: model.RuleCategories[model.CategorySchemas],
+		Recommended:  true,
+		Type:         Style,
+		Severity:     model.SeverityWarn,
+		Then: model.RuleAction{
+			Function: "oasUnnecessaryCombinator",
+		},
+		HowToFix: unnecessaryCombinatorFix,
+	}
+}
+
+// GetCamelCasePropertiesRule will check that all schema property names are in camelCase format.
+func GetCamelCasePropertiesRule() *model.Rule {
+	return &model.Rule{
+		Name:         "Check schema property names are camelCase",
+		Id:           CamelCasePropertiesRule,
+		Formats:      model.OAS3AllFormat,
+		Description:  "Schema property names should use camelCase instead of snake_case, PascalCase, kebab-case, or other formats.",
+		Given:        "$",
+		Resolved:     true,
+		RuleCategory: model.RuleCategories[model.CategorySchemas],
+		Recommended:  true,
+		Type:         Style,
+		Severity:     model.SeverityWarn,
+		Then: model.RuleAction{
+			Function: "oasCamelCaseProperties",
+		},
+		HowToFix: camelCasePropertiesFix,
+	}
+}
+
+// GetMigrateZallyIgnoreRule will check for x-zally-ignore keys and suggest migration to x-lint-ignore
+func GetMigrateZallyIgnoreRule() *model.Rule {
+	return &model.Rule{
+		Name:         "Migrate from x-zally-ignore to x-lint-ignore",
+		Id:           MigrateZallyIgnoreRule,
+		Formats:      model.AllFormats,
+		Description:  "x-zally-ignore keys should be migrated to x-lint-ignore for compatibility with vacuum",
+		Given:        "$",
+		Resolved:     true,
+		RuleCategory: model.RuleCategories[model.CategoryValidation],
+		Recommended:  true,
+		Type:         Validation,
+		Severity:     model.SeverityWarn,
+		Then: model.RuleAction{
+			Function: "migrateZallyIgnore",
+		},
+		HowToFix: migrateZallyIgnoreFix,
+	}
+}
