@@ -46,7 +46,7 @@ func TestCreateReportStatistics_AlmostPerfect(t *testing.T) {
 
 	//assert.Equal(t, 100, stats.OverallScore)
 	// new missing examples function is now strict / correct
-	assert.Equal(t, 97, stats.OverallScore)
+	assert.GreaterOrEqual(t, stats.OverallScore, 98)
 
 }
 
@@ -56,7 +56,7 @@ func TestCreateReportStatistics_BigLoadOfIssues(t *testing.T) {
 	selectedRS := defaultRuleSets.GenerateOpenAPIRecommendedRuleSet()
 	specBytes, _ := os.ReadFile("../model/test_files/api.github.com.yaml")
 
-	ctx, cancel := context.WithTimeout(context.Background(), time.Second*20)
+	ctx, cancel := context.WithTimeout(context.Background(), time.Second*30)
 	defer cancel()
 	d := make(chan bool)
 	go func(f chan bool) {
@@ -79,5 +79,4 @@ func TestCreateReportStatistics_BigLoadOfIssues(t *testing.T) {
 	case <-d:
 		break
 	}
-
 }
